"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComponentParser = void 0;
const change_case_1 = require("change-case");
const path_1 = require("path");
const types_1 = require("./helpers/types");
const ELEMENT_PREFIX = 'el';
const CODE_FILE = 'codeFile';
const CSS_FILE = 'cssFile';
const MULTI_TEMPLATE_TAG = 'template';
const MULTI_TEMPLATE_KEY_ATTRIBUTE = 'key';
const KNOWN_TEMPLATE_SUFFIX = 'Template';
const KNOWN_MULTI_TEMPLATE_SUFFIX = 'Templates';
const knownCollections = ['items', 'spans', 'actionItems'];
const knownPlatforms = ['android', 'ios', 'desktop'];
class ComponentParser {
    constructor(moduleRelativePath, platform) {
        this.parentIndices = new Array();
        this.complexProperties = new Array();
        this.resolvedRequests = new Array();
        // Keep counter for the case of platform tags being inside platform tags
        this.unsupportedPlatformTagCount = 0;
        this.moduleDirPath = '';
        this.moduleRelativePath = '';
        this.head = '';
        this.body = '';
        this.treeIndex = 0;
        const { dir, ext, name } = (0, path_1.parse)(moduleRelativePath);
        const componentName = (0, change_case_1.pascalCase)(name);
        this.moduleDirPath = dir;
        this.moduleRelativePath = moduleRelativePath.substring(0, moduleRelativePath.length - ext.length);
        this.appendImports();
        this.body += `export default class ${componentName} {
      constructor() {`;
        this.platform = platform;
    }
    handleOpenTag(elementName, prefix, attributes) {
        var _a;
        // Platform tags
        if (knownPlatforms.includes(elementName)) {
            if (elementName.toLowerCase() !== this.platform) {
                this.unsupportedPlatformTagCount++;
            }
            return;
        }
        if (this.unsupportedPlatformTagCount > 0) {
            return;
        }
        const parentIndex = (_a = this.parentIndices[this.parentIndices.length - 1]) !== null && _a !== void 0 ? _a : -1;
        if (elementName === MULTI_TEMPLATE_TAG) {
            const complexProperty = this.complexProperties[this.complexProperties.length - 1];
            if (parentIndex >= 0 && complexProperty && complexProperty.parentIndex == parentIndex) {
                if (attributes[MULTI_TEMPLATE_KEY_ATTRIBUTE]) {
                    // This is necessary for proper string escape
                    const attrValue = attributes[MULTI_TEMPLATE_KEY_ATTRIBUTE].value.replaceAll('\'', '\\\'');
                    this.body += `{ key: '${attrValue}', createView: () => {`;
                }
                else {
                    // eslint-disable-next-line no-console
                    console.warn('Found template without key inside ${complexProperty.name}');
                }
            }
        }
        else if (this.isComplexProperty(elementName)) {
            const complexProperty = {
                parentIndex,
                name: this.getComplexPropertyName(elementName),
                elementReferences: [],
                templateViewIndex: this.treeIndex
            };
            this.complexProperties.push(complexProperty);
            this.body += `/* ${elementName} - start */`;
            if (complexProperty.name.endsWith(KNOWN_TEMPLATE_SUFFIX)) {
                this.body += `${ELEMENT_PREFIX}${parentIndex}.${complexProperty.name} = () => {`;
            }
            else if (complexProperty.name.endsWith(KNOWN_MULTI_TEMPLATE_SUFFIX)) {
                this.body += `${ELEMENT_PREFIX}${parentIndex}.${complexProperty.name} = [`;
            }
        }
        else {
            const complexProperty = this.complexProperties[this.complexProperties.length - 1];
            this.checkForNamespaces(attributes);
            this.buildComponent(elementName, prefix, attributes);
            if (parentIndex >= 0) {
                if (complexProperty && complexProperty.parentIndex == parentIndex) {
                    // Add component to complex property of parent component
                    this.addToComplexProperty(parentIndex, complexProperty);
                }
                else {
                    this.body += `${ELEMENT_PREFIX}${parentIndex}._addChildFromBuilder && ${ELEMENT_PREFIX}${parentIndex}._addChildFromBuilder('${elementName}', ${ELEMENT_PREFIX}${this.treeIndex});`;
                }
            }
            this.parentIndices.push(this.treeIndex);
            this.treeIndex++;
        }
    }
    handleCloseTag(elementName) {
        var _a;
        // Platform tags
        if (knownPlatforms.includes(elementName)) {
            if (elementName.toLowerCase() !== this.platform) {
                this.unsupportedPlatformTagCount--;
            }
            return;
        }
        if (this.unsupportedPlatformTagCount > 0) {
            return;
        }
        const parentIndex = (_a = this.parentIndices[this.parentIndices.length - 1]) !== null && _a !== void 0 ? _a : -1;
        const complexProperty = this.complexProperties[this.complexProperties.length - 1];
        if (elementName === MULTI_TEMPLATE_TAG) {
            if (parentIndex >= 0 && complexProperty && complexProperty.parentIndex == parentIndex) {
                this.body += this.treeIndex > complexProperty.templateViewIndex ? `return ${ELEMENT_PREFIX}${complexProperty.templateViewIndex}; }},` : 'return null; }},';
                complexProperty.templateViewIndex = this.treeIndex;
            }
        }
        else if (this.isComplexProperty(elementName)) {
            if (complexProperty) {
                if (complexProperty.name.endsWith(KNOWN_TEMPLATE_SUFFIX)) {
                    this.body += this.treeIndex > complexProperty.templateViewIndex ? `return ${ELEMENT_PREFIX}${complexProperty.templateViewIndex}; };` : 'return null; };';
                }
                else if (complexProperty.name.endsWith(KNOWN_MULTI_TEMPLATE_SUFFIX)) {
                    this.body += '];';
                }
                else if (parentIndex >= 0) {
                    // If parent is AddArrayFromBuilder call the interface method to populate the array property
                    this.body += `${ELEMENT_PREFIX}${parentIndex}._addArrayFromBuilder && ${ELEMENT_PREFIX}${parentIndex}._addArrayFromBuilder('${complexProperty.name}', [${complexProperty.elementReferences.join(', ')}]);`;
                    complexProperty.elementReferences = [];
                }
            }
            this.body += `/* ${elementName} - end */`;
            // Remove the last complexProperty from the complexProperties collection (move to the previous complexProperty scope)
            this.complexProperties.pop();
        }
        else {
            // Remove the last parent from the parents collection (move to the previous parent scope)
            this.parentIndices.pop();
        }
    }
    getResolvedRequests() {
        return this.resolvedRequests;
    }
    finish() {
        this.body += `return ${ELEMENT_PREFIX}0;
      }
    }`;
    }
    getOutput() {
        return this.head + this.body;
    }
    appendImports() {
        this.head += `var { resolveModuleName } = require('@nativescript/core/module-name-resolver');
    var uiCoreModules = require('@nativescript/core/ui');
    var { setPropertyValue } = require('@nativescript/core/ui/builder/component-builder');
    var customModules = {};
    `;
    }
    applyComponentAttributes(attributes) {
        const attributeData = Object.values(attributes);
        for (const { local, prefix, value } of attributeData) {
            // Namespaces are not regarded as properties
            if (prefix === 'xmlns') {
                continue;
            }
            // Platform-based attributes
            if (knownPlatforms.includes(prefix.toLowerCase()) && prefix.toLowerCase() !== this.platform.toLowerCase()) {
                continue;
            }
            // This is necessary for proper string escape
            let propertyName = local;
            const attrValue = value.replaceAll('\'', '\\\'');
            let instanceReference = `${ELEMENT_PREFIX}${this.treeIndex}`;
            if (propertyName.indexOf('.') !== -1) {
                const properties = propertyName.split('.');
                for (let i = 0, length = properties.length - 1; i < length; i++) {
                    instanceReference += `?.${properties[i]}`;
                }
                propertyName = properties[properties.length - 1];
            }
            this.body += `${instanceReference} && setPropertyValue(${instanceReference}, null, moduleExports, '${propertyName}', '${attrValue}');`;
        }
    }
    buildComponent(elementName, prefix, attributes) {
        this.body += `var ${ELEMENT_PREFIX}${this.treeIndex} = global.xmlCompiler.newInstance('${elementName}', '${prefix}', uiCoreModules, customModules);`;
        if (this.treeIndex == 0) {
            // Script
            if (attributes[CODE_FILE]) {
                const attrValue = attributes[CODE_FILE].value;
                this.resolvedRequests.push(attrValue);
                const resolvedPath = this.getResolvedPath(attrValue);
                this.body += `var resolvedCodeModuleName = resolveModuleName('${resolvedPath}', '');`;
            }
            else {
                this.body += `var resolvedCodeModuleName = resolveModuleName('${this.moduleRelativePath}', '');
        if (!resolvedCodeModuleName && this.__fallbackModuleRelativePath) {
          resolvedCodeModuleName = resolveModuleName(this.__fallbackModuleRelativePath, '');
        }`;
            }
            this.body += 'var moduleExports = resolvedCodeModuleName ? global.loadModule(resolvedCodeModuleName, true) : null;';
            // Style
            if (attributes[CSS_FILE]) {
                const attrValue = attributes[CSS_FILE].value;
                this.resolvedRequests.push(attrValue);
                const resolvedPath = this.getResolvedPath(attrValue);
                this.body += `var resolvedCssModuleName = resolveModuleName('${resolvedPath}', 'css');`;
            }
            else {
                this.body += `var resolvedCssModuleName = resolveModuleName('${this.moduleRelativePath}', 'css');`;
            }
            this.body += `resolvedCssModuleName && ${ELEMENT_PREFIX}${this.treeIndex}.addCssFile(resolvedCssModuleName);`;
        }
        this.applyComponentAttributes(attributes);
    }
    checkForNamespaces(attributes) {
        const attributeData = Object.values(attributes);
        /**
         * By default, virtual-entry-javascript registers all application js, xml, and css files as modules.
         * Registering namespaces will ensure node modules are also included in module register.
         * However, we have to ensure that the resolved path of files is used as module key so that module-name-resolver works properly.
         */
        for (const { local, prefix, value } of attributeData) {
            if (local && prefix === 'xmlns') {
                this.resolvedRequests.push(value);
                const resolvedPath = this.getResolvedPath(value);
                const ext = resolvedPath.endsWith('.xml') ? 'xml' : '';
                // Register module using resolve path as key and overwrite old registration if any
                this.head += `global.registerModule('${resolvedPath}', () => require('${value}'));`;
                this.body += `global.xmlCompiler.loadCustomModule('${local}', '${resolvedPath}', '${ext}', customModules);`;
            }
        }
    }
    getResolvedPath(uri) {
        return uri.startsWith('~/') ? uri.substr(2) : (0, path_1.join)(this.moduleDirPath, uri);
    }
    addToComplexProperty(parentIndex, complexProperty) {
        // If property name is known collection we populate array with elements
        if (knownCollections.includes(complexProperty.name)) {
            complexProperty.elementReferences.push(`${ELEMENT_PREFIX}${this.treeIndex}`);
        }
        else if (complexProperty.name.endsWith(KNOWN_TEMPLATE_SUFFIX) || complexProperty.name.endsWith(KNOWN_MULTI_TEMPLATE_SUFFIX)) {
            // Do nothing
        }
        else {
            // Add child parent else simply assign it as a value
            this.body += `if (${ELEMENT_PREFIX}${parentIndex}._addChildFromBuilder) {
        ${ELEMENT_PREFIX}${parentIndex}._addChildFromBuilder('${complexProperty.name}', ${ELEMENT_PREFIX}${this.treeIndex});
      } else {
        ${ELEMENT_PREFIX}${parentIndex}['${complexProperty.name}'] = ${ELEMENT_PREFIX}${this.treeIndex};
      }`;
        }
    }
    getComplexPropertyName(fullName) {
        let name;
        if ((0, types_1.isString)(fullName)) {
            const names = fullName.split('.');
            name = names[names.length - 1];
        }
        return name;
    }
    isComplexProperty(name) {
        return (0, types_1.isString)(name) && name.indexOf('.') !== -1;
    }
}
exports.ComponentParser = ComponentParser;
//# sourceMappingURL=component-parser.js.map